/*
 * Memory region abstractions.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <string.h>
#include <stdlib.h>

#include "region.h"



/* Initialise region state, indicating the size, file and position. */

RegionState::RegionState(unsigned long size, fileid_t fileid, offset_t filepos)
: size(size), fileid(fileid), filepos(filepos)
{
}

void RegionState::fill(fileid_t fileid, offset_t filepos)
{
    this->fileid = fileid;
    this->filepos = filepos;
}



/* Initialise a region having the given 'start' and 'end' addresses, with the
   'end' being one location beyond the last address in the region. */

Region::Region(offset_t start, offset_t end)
: start(start), end(end), state(end - start)
{
    /* Content state. */

    memset((void *) start, 0, end - start);
}

Region::~Region()
{
    free((void *) start);
}

offset_t Region::size()
{
    return end - start;
}

/* Debugging methods. */

int Region::compare(Region *other)
{
    if (start < other->start)
        return -1;
    else if (start > other->start)
        return 1;
    else
        return 0;
}

void Region::fill(fileid_t fileid, offset_t filepos)
{
    state.fill(fileid, filepos);
}

void Region::flush()
{
}

/* Simulation methods. */

char *Region::read(offset_t offset)
{
    if (offset < size())
        return (char *) start + offset;
    else
        return NULL;
}

void Region::write(const char *data, offset_t offset)
{
    size_t length = strlen(data);

    if (offset + length < size())
        memcpy((void *) (start + offset), data, length + 1);
}

// vim: tabstop=4 expandtab shiftwidth=4
